/**
 * File:    CombinedImageDescriptor.java
 * Created: 22.03.2006
 *
 *
 * Copyright (c) 2006  University of Augsburg <www.informatik.uni-augsburg.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 */


package de.uniAugsburg.agilPro.figures;


import java.util.Arrays;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Device;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;




/**
 * ImageDescriptor that uses several other ImageDescriptors to create the image.
 * 
 * <p>All ImageDescriptors are asked in sequence. The first image that is not
 * <code>null</code> is returned.</p>
 * 
 * @version $Id: CombinedImageDescriptor.java 353 2006-05-26 16:40:06Z bauermar $
 * @author Markus Bauer (markusbauer@users.sourceforge.net)
 */
public class CombinedImageDescriptor
    extends ImageDescriptor
{
  /**
   * The ImageDescriptors.
   * 
   * <p>
   * Never <code>null</code>.
   * </p>
   */
  private ImageDescriptor[] imageDescriptors;




  /**
   * @param imageDescriptors The ImageDescriptors.
   */
  public CombinedImageDescriptor(ImageDescriptor[] imageDescriptors)
  {
    assert imageDescriptors != null;
    this.imageDescriptors = imageDescriptors;
  }



  /**
   * @return Returns the imageDescriptors.
   */
  public ImageDescriptor[] getImageDescriptors()
  {
    return imageDescriptors;
  }



  /**
   * Asks all {@link #imageDescriptors} to create an image.
   * 
   * @see org.eclipse.jface.resource.ImageDescriptor#createImage(boolean,
   *      org.eclipse.swt.graphics.Device)
   */
  @Override
  public Image createImage(boolean returnMissingImageOnError, Device device)
  {
    for (ImageDescriptor imageDescriptor : imageDescriptors)
    {
      if (imageDescriptor != null)
      {
        Image image;
        try
        {
          image = imageDescriptor.createImage(false, device);
          if (image != null)
          {
            return image;
          }
        }
        catch (RuntimeException e)
        {
          // create failed, try next descriptor
        }
      }
    }

    if (returnMissingImageOnError)
    {
      return getMissingImageDescriptor().createImage(true, device);
    }

    return null;
  }




  /**
   * Asks all {@link #imageDescriptors} to create the image data.
   * 
   * @see org.eclipse.jface.resource.ImageDescriptor#getImageData()
   */
  @Override
  public ImageData getImageData()
  {
    for (ImageDescriptor imageDescriptor : imageDescriptors)
    {
      if (imageDescriptor != null)
      {
        ImageData imageData;

        try
        {
          imageData = imageDescriptor.getImageData();
          if (imageData != null)
          {
            return imageData;
          }
        }
        catch (Exception e)
        {
          // failed, try next descriptor
        }
      }
    }

    return null;
  }



  /* (non-Javadoc)
   * @see java.lang.Object#toString()
   */
  @Override
  public String toString()
  {
    return Arrays.toString(imageDescriptors);
  }
}
