/**
 * File:    ImageFactory.java
 * Created: 25.02.2006
 *
 *
 * Copyright (c) 2006  University of Augsburg <www.informatik.uni-augsburg.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 */


package de.uniAugsburg.agilPro.figures;


import java.util.MissingResourceException;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.ui.plugin.AbstractUIPlugin;

/**
 * Creates images. It uses an {@link ImageRegistry} for disposing the images.
 * 
 * @version $Id: ImageFactory.java 566 2006-09-28 11:20:57Z fischewo $
 * @author Markus Bauer <markusbauer@users.sourceforge.net>
 */
public class ImageFactory
{


  /**
   * Path where the icons are located.
   */
  public static final String ICONS_BASE_PATH = "icons/";


  /**
   * The size for images representing a model type.
   */
  public static final Dimension MODEL_TYPE_IMAGE_SIZE = new Dimension(16, 16);


  /**
   * A ImageRegistry that handles images.
   * 
   * <p>
   * Never <code>null</code>.
   * </p>
   */
  private ImageRegistry imageRegistry;




  /**
   * @param imageRegistry The registry for disposing the images.
   */
  public ImageFactory(ImageRegistry imageRegistry)
  {
    assert imageRegistry != null;
    this.imageRegistry = imageRegistry;
  }



  /**
   * Returns an image.
   * 
   * <p>
   * The {@link ImageRegistry} handles dispose of the image.
   * </p>
   * 
   * @param imageDescriptor The {@link ImageDescriptor} that creates the image.
   *          Its String representations is the key in the {@link ImageRegistry}.
   * @return The image, or <code>null</code> if it could not be found.
   */
  public Image getImage(ImageDescriptor imageDescriptor)
  {
    return getImage(imageDescriptor, null);
  }



  /**
   * Returns an image from the {@link ImageRegistry}.
   * 
   * <p>
   * The image will be scaled to the given size.
   * </p>
   * 
   * @param imageDescriptor The {@link ImageDescriptor} that creates the image.
   *          Its String representations is the key in the {@link ImageRegistry}.
   * @param size The size of the returned image. If <code>null</code> the
   *          image is returned unscaled.
   * @return The image, or <code>null</code> if it could not be found.
   */
  public Image getImage(ImageDescriptor imageDescriptor, Dimension size)
  {
    if (imageDescriptor == null)
    {
      return null;
    }

    // scale the image if necessary
    
    imageDescriptor = new ScalingImageDescriptor(imageDescriptor, size);
    String name = imageDescriptor.toString();

    Image image = imageRegistry.get(name);
    
    if (image != null)
    {
//      logger.debug("Image found in registry as: " + name);
    }
    else
    {
      // load new image into the registry
      imageRegistry.put(name, imageDescriptor);

      image = imageRegistry.get(name);
    }

    return image;
  }



  /**
   * Returns an image descriptor for the image file at the given plug-in
   * relative path.
   * 
   * @param path the path
   * @return the image descriptor
   */
  public static ImageDescriptor createImageDescriptor(String path)
  {
	String temppath = ICONS_BASE_PATH + path;
    ImageDescriptor result = AbstractUIPlugin.imageDescriptorFromPlugin("",
                                                      temppath);
    /**
     * If the icon could not be found (for example because the user put it in a different
     * folder than icons, load it a snd time but this time without the icons folder prefix
     */
    if (result == null) result = AbstractUIPlugin.imageDescriptorFromPlugin("",
            path);
    return result;
  }



  /**
   * Creates an ImageDescriptor that represents the type of a model object.
   * 
   * @param model The model object.
   * @return The ImageDescriptor.
   */
  public ImageDescriptor createModelTypeImageDescriptor(Object model)
  {
    String path;
    try
    {
      path = "";//PluginProperties.model_smallIcon(model);
    }
    catch (MissingResourceException e)
    {
      return null;
    }

    ImageDescriptor imageDescriptor = createImageDescriptor(path);

    // test if the image is available
    if (imageDescriptor == null) return null;
    ImageData data = imageDescriptor.getImageData();

    return data == null ? null
                       : new ScalingImageDescriptor(imageDescriptor,
                                                    MODEL_TYPE_IMAGE_SIZE);
  }
}
