/**
 * File:    ScalingImageDescriptor.java
 * Created: 22.03.2006
 *
 *
 * Copyright (c) 2006  University of Augsburg <www.informatik.uni-augsburg.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 */


package de.uniAugsburg.agilPro.figures;


import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.ImageData;




/**
 * This ImageDescriptor scales the image returned by another ImageDescriptor.
 * 
 * @version $Id: ScalingImageDescriptor.java 353 2006-05-26 16:40:06Z bauermar $
 * @author Markus Bauer <markusbauer@users.sourceforge.net>
 */
public class ScalingImageDescriptor
    extends ImageDescriptor
{
  /**
   * The ImageDescriptor that creates the original image.
   */
  private ImageDescriptor imageDescriptor;


  /**
   * The size of the returned image.
   * 
   * <p>
   * If <code>null</code> the size of the image is not changed.
   * </p>
   */
  private Dimension size;




  /**
   * Creates a new image descriptor.
   * 
   * @param imageDescriptor The ImageDescriptor that creates the original image.
   * @param size The size of the returned image.
   */
  public ScalingImageDescriptor(ImageDescriptor imageDescriptor, Dimension size)
  {
    setSize(size);
    setImageDescriptor(imageDescriptor);
  }



  /**
   * @return Returns the imageDescriptor.
   */
  public ImageDescriptor getImageDescriptor()
  {
    return imageDescriptor;
  }



  /**
   * @param imageDescriptor The imageDescriptor to set.
   */
  public void setImageDescriptor(ImageDescriptor imageDescriptor)
  {
    this.imageDescriptor = imageDescriptor;
    
    // optimisation: dont scale image twice
    if (imageDescriptor != null)
    {
      if (imageDescriptor instanceof ScalingImageDescriptor)
      {
        this.imageDescriptor =
            ((ScalingImageDescriptor) imageDescriptor).getImageDescriptor();
      }

      // disable scaling of nested ScalingImageDescriptors
      if (imageDescriptor instanceof CombinedImageDescriptor)
      {
        CombinedImageDescriptor combinedImageDescriptor =
            (CombinedImageDescriptor) imageDescriptor;

        for (ImageDescriptor descriptor : combinedImageDescriptor
            .getImageDescriptors())
        {
          if (descriptor != null
              && descriptor instanceof ScalingImageDescriptor)
          {
            ((ScalingImageDescriptor) descriptor).setSize(null);
          }
        }
      }
    }
  }



  /**
   * @return Returns the size.
   */
  public Dimension getSize()
  {
    return size;
  }



  /**
   * @param size The size to set.
   */
  public void setSize(Dimension size)
  {
    this.size = size;
  }



  /* (non-Javadoc)
   * @see org.eclipse.jface.resource.ImageDescriptor#getImageData()
   */
  @Override
  public ImageData getImageData()
  {
    if (imageDescriptor == null)
    {
      return null;
    }

    ImageData imageData = imageDescriptor.getImageData();

    if (imageData == null)
    {
      return null;
    }

    if (size != null
        && (imageData.width != size.width || imageData.height != size.height))
    {
      imageData = imageData.scaledTo(size.width, size.height);
    }

    return imageData;
  }



  /* (non-Javadoc)
   * @see java.lang.Object#toString()
   */
  @Override
  public String toString()
  {
    if (size == null)
    {
      return imageDescriptor + "";
    }

    return imageDescriptor + "@" + size.width + "x" + size.height;
  }
}
